#!/usr/bin/perl -w
# -*- perl -*-

use strict;

use FindBin;
use lib "$FindBin::Bin";

our @inttypes;
our @floattypes;
our @includes;
our %types;
require 'types.pl';
require 'common.pl';

$\ = "\n";

print '<%# /* -*- c -*- */';
print '#%>';
print '#ifndef __P_ACCESSOR_H';
print '#define __P_ACCESSOR_H';
print '';
print '/* <%=s EC_VERSION_MSG %> */';

foreach((map{ ("signed $_", "unsigned $_") } @inttypes), @floattypes){
  my $t = $_;
  my $ut = $t; $ut =~ s/ /_/g;

  print '';
  print "<% if( sizeof($t) < sizeof(void*) || ($t)0.1 != 0.){ %>";
  # not converted
  print &gen_typedef($t, $ut, $t);
  print &gen_wrap   ($t, $ut, 'return (v);');
  print &gen_unwrap ($t, $ut, 'return (v);');
  print &gen_param  ($t, $ut, "$t n");
  print &gen_member ($t, $ut, '(&(t)->m)');
  print &gen_read   ($t, $ut, 'return *p;');
  print &gen_write  ($t, $ut, '*p = v;');
  print &gen_read_unwrap ($t, $ut);
  print &gen_wrap_write  ($t, $ut);

  print "<% if( sizeof($t) == 1 ){ %>";
  print &gen_read_a ($t, $ut, '(read_byte(p##_base, p##_ofs + (n)))');
  print &gen_write_a($t, $ut, '(write_byte(p##_base, p##_ofs + (n), (v), NULL))');
  print "<% }else if( sizeof($t) * 2 == sizeof(void*) ){ %>";
  print &gen_read_a ($t, $ut, '(read_hword(p##_base, p##_ofs + (n) * (sizeof(void*))/2))');
  print &gen_write_a($t, $ut, '(write_hword(p##_base, p##_ofs + (n) * (sizeof(void*)/2), (v), NULL))');
  print '<% }else{ %>';
  print "/* read-write array of '$t' not supported.*/";
  print "<% } %>";

  print "<% }else if( sizeof($t) == sizeof(void*) ){ %>";

  # converted to value in fsc
  print &gen_typedef($t, $ut, 'value');
  print &gen_wrap   ($t, $ut, 'return value_of_base_vaddr(0, v);');
  print &gen_unwrap ($t, $ut, 'return vaddr_of_value(v);');
  print &gen_param  ($t, $ut, "base_t n##_b, $t n");
  print &gen_member ($t, $ut, '(&((t)->m).cv)');
  print &gen_read   ($t, $ut, 'return *p;');
  print &gen_write  ($t, $ut, '*p = v;');
  print &gen_read_unwrap ($t, $ut);
  print &gen_wrap_write  ($t, $ut);
  print &gen_read_a ($t, $ut, '(read_word(p##_base, p##_ofs + (n) * sizeof(void*)))');
  print &gen_write_a($t, $ut, '(write_word(p##_base, p##_ofs + (n) * sizeof(void*), (v), NULL))');

  print "<% }else if( sizeof($t) <= sizeof(void*)*2 ){ %>";
  # converted to dvalue in fsc
  print &gen_typedef($t, $ut, 'dvalue');
  print &gen_wrap   ($t, $ut, 'return dvalue_of_base_vaddr(0, v);');
  print &gen_unwrap ($t, $ut, 'return vaddr_of_dvalue(v);');
  print &gen_param  ($t, $ut, "base_t n##_b, $t n");
  print &gen_member ($t, $ut, '(&((t)->m).cv)');
  print &gen_read   ($t, $ut, 'return *p;');
  print &gen_write  ($t, $ut, '*p = v;');
  print &gen_read_unwrap ($t, $ut);
  print &gen_wrap_write  ($t, $ut);
  print &gen_read_a ($t, $ut, '(read_dword(p##_base, p##_ofs + (n) * sizeof(void*)))');
  print &gen_write_a($t, $ut, '(write_dword(p##_base, p##_ofs + (n) * sizeof(void*), (v), NULL))');

  print "<% }else{ %>";
  # not supported in fsc
  print "/* '$t' is not supported in wrapper */";
  print "<% } %>"
}

sub fscw_chain {
  my $b = shift; my $ub = $b; $ub =~ s/ /_/g;
  my $t = shift; my $ut = $t; $ut =~ s/ /_/g;

  print &gen_typedef($t, $ut, &gen_name($b, $ub));
  print &gen_wrap   ($t, $ut, 'return ' . &gen_name($b, $ub, 'wrap') . '(v);');
  print &gen_unwrap ($t, $ut, 'return ' . &gen_name($b, $ub, 'unwrap') . '(v);');
  print &gen_param  ($t, $ut, &gen_name($b, $ub, 'param') . '(n)');
  print &gen_member ($t, $ut, &gen_name($b, $ub, 'member') . '(t, m)');
  print &gen_read   ($t, $ut, 'return ' . &gen_name($b, $ub, 'read') . '(p);');
  print &gen_write  ($t, $ut, &gen_name($b, $ub, 'write') . '(p, v);');
  print &gen_read_unwrap ($t, $ut);
  print &gen_wrap_write  ($t, $ut);
  print &gen_read_a ($t, $ut, &gen_name($b, $ub, 'read_a') . '(p, n)');
  print &gen_write_a($t, $ut, &gen_name($b, $ub, 'write_a') . '(p, n, v)');
}

foreach(@inttypes){
  print '';

  if($_ eq 'char'){
    print '<% if( (char)-1 < 0 ){ %>';
    &fscw_chain("signed $_", $_);
    print '<% }else{ %>';
    &fscw_chain("unsigned $_", $_);
    print '<% } %>';
  }else{
    &fscw_chain("signed $_", $_);
  }
}

print '';
print '#endif /*  __P_ACCESSOR_H */';
