/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/cbrtf.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname cbrtf_1
  * @testfor cbrtf
  */
TEST_CASE(cbrtf_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(float)];
    float v;
  } result, args[] = {
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x80, 0x00, },
      /* 1.175494e-38 */
    {0x00, 0x00, 0x80, 0x80, },
      /* -1.175494e-38 */
    {0xff, 0xff, 0x7f, 0x7f, },
      /* 3.402823e+38 */
    {0xff, 0xff, 0x7f, 0xff, },
      /* -3.402823e+38 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0xff, },
      /* -inf */
    {0x00, 0x00, 0xc0, 0x7f, },
      /* nan */
    {0x00, 0x00, 0xc0, 0xff, },
      /* nan */
    {0x66, 0x66, 0x66, 0x3f, },
      /* 9.000000e-01 */
    {0xcd, 0xcc, 0x8c, 0x3f, },
      /* 1.100000e+00 */
    {0x66, 0x66, 0x66, 0xbf, },
      /* -9.000000e-01 */
    {0xcd, 0xcc, 0x8c, 0xbf, },
      /* -1.100000e+00 */
  }, expected[] = {
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x80, 0x2a, },
      /* 2.273737e-13 */
    {0x00, 0x00, 0x80, 0xaa, },
      /* -2.273737e-13 */
    {0xf5, 0x2f, 0xcb, 0x54, },
      /* 6.981464e+12 */
    {0xf5, 0x2f, 0xcb, 0xd4, },
      /* -6.981464e+12 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0xff, },
      /* -inf */
    {0x00, 0x00, 0xc0, 0x7f, },
      /* nan */
    {0x00, 0x00, 0xc0, 0xff, },
      /* nan */
    {0x50, 0x2a, 0x77, 0x3f, },
      /* 9.654894e-01 */
    {0xc1, 0x21, 0x84, 0x3f, },
      /* 1.032280e+00 */
    {0x50, 0x2a, 0x77, 0xbf, },
      /* -9.654894e-01 */
    {0xc1, 0x21, 0x84, 0xbf, },
      /* -1.032280e+00 */
  };

  for (i = 0; i < 16; ++i) {
    result.v = cbrtf( args[i * 1 + 0].v);
    if (memcmp(result.b, expected[i].b, sizeof(float)) != 0) {
      int j;
      fprintf(stderr, "\nreturned: ");
      for (j = 0; j < sizeof(float); ++j) {
        fprintf(stderr, "%02x, ", result.b[j]);
      }
      fprintf(stderr, "(%e)\n", result.v);
      fprintf(stderr, "expected: ");
      for (j = 0; j < sizeof(float); ++j) {
        fprintf(stderr, "%02x, ", expected[i].b[j]);
      }
      fprintf(stderr, "(%e)\n", expected[i].v);
      ok = 0;
    }
  }

  TEST(ok);
}
