/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/cosf.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname cosf_1
  * @testfor cosf
  */
TEST_CASE(cosf_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(float)];
    float v;
  } result, args[] = {
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x80, 0x00, },
      /* 1.175494e-38 */
    {0x00, 0x00, 0x80, 0x80, },
      /* -1.175494e-38 */
    {0xff, 0xff, 0x7f, 0x7f, },
      /* 3.402823e+38 */
    {0xff, 0xff, 0x7f, 0xff, },
      /* -3.402823e+38 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0xff, },
      /* -inf */
    {0x00, 0x00, 0xc0, 0x7f, },
      /* nan */
    {0x00, 0x00, 0xc0, 0xff, },
      /* nan */
    {0xdb, 0x0f, 0xc9, 0x3f, },
      /* 1.570796e+00 */
    {0xdb, 0x0f, 0x49, 0x40, },
      /* 3.141593e+00 */
    {0xe4, 0xcb, 0x96, 0x40, },
      /* 4.712389e+00 */
    {0xdb, 0x0f, 0xc9, 0x40, },
      /* 6.283185e+00 */
    {0xdb, 0x0f, 0xc9, 0xbf, },
      /* -1.570796e+00 */
    {0xdb, 0x0f, 0x49, 0xc0, },
      /* -3.141593e+00 */
    {0xe4, 0xcb, 0x96, 0xc0, },
      /* -4.712389e+00 */
    {0xdb, 0x0f, 0xc9, 0xc0, },
      /* -6.283185e+00 */
    {0xdb, 0x0f, 0x49, 0x3f, },
      /* 7.853982e-01 */
    {0xdb, 0x0f, 0x49, 0xbf, },
      /* -7.853982e-01 */
  }, expected[] = {
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x40, 0x51, 0x0a, 0x3f, },
      /* 5.403023e-01 */
    {0x40, 0x51, 0x0a, 0x3f, },
      /* 5.403023e-01 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x72, 0x14, 0x5b, 0xbe, },
      /* -2.139452e-01 */
    {0x72, 0x14, 0x5b, 0xbe, },
      /* -2.139452e-01 */
    {0x00, 0x00, 0xc0, 0xff, },
      /* nan */
    {0x00, 0x00, 0xc0, 0xff, },
      /* nan */
    {0x00, 0x00, 0xc0, 0x7f, },
      /* nan */
    {0x00, 0x00, 0xc0, 0xff, },
      /* nan */
    {0x2e, 0xbd, 0x3b, 0xb3, },
      /* -4.371139e-08 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x2e, 0xde, 0x4c, 0x32, },
      /* 1.192488e-08 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x2e, 0xbd, 0x3b, 0xb3, },
      /* -4.371139e-08 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x2e, 0xde, 0x4c, 0x32, },
      /* 1.192488e-08 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0xf3, 0x04, 0x35, 0x3f, },
      /* 7.071068e-01 */
    {0xf3, 0x04, 0x35, 0x3f, },
      /* 7.071068e-01 */
  };

  for (i = 0; i < 22; ++i) {
    result.v = cosf( args[i * 1 + 0].v);
    if (memcmp(result.b, expected[i].b, sizeof(float)) != 0) {
      int j;
      fprintf(stderr, "\nreturned: ");
      for (j = 0; j < sizeof(float); ++j) {
        fprintf(stderr, "%02x, ", result.b[j]);
      }
      fprintf(stderr, "(%e)\n", result.v);
      fprintf(stderr, "expected: ");
      for (j = 0; j < sizeof(float); ++j) {
        fprintf(stderr, "%02x, ", expected[i].b[j]);
      }
      fprintf(stderr, "(%e)\n", expected[i].v);
      ok = 0;
    }
  }

  TEST(ok);
}
