/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/erf.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname erf_1
  * @testfor erf
  */
TEST_CASE(erf_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(double)];
    double v;
  } result, args[] = {
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, },
      /* 2.225074e-308 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, },
      /* -2.225074e-308 */
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0x7f, },
      /* 1.797693e+308 */
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0xff, },
      /* -1.797693e+308 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xff, },
      /* -inf */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x7f, },
      /* nan */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xff, },
      /* nan */
    {0xcd, 0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0xec, 0x3f, },
      /* 9.000000e-01 */
    {0x9a, 0x99, 0x99, 0x99, 0x99, 0x99, 0xf1, 0x3f, },
      /* 1.100000e+00 */
    {0xcd, 0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0xec, 0xbf, },
      /* -9.000000e-01 */
    {0x9a, 0x99, 0x99, 0x99, 0x99, 0x99, 0xf1, 0xbf, },
      /* -1.100000e+00 */
  }, expected[] = {
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x8b, 0x08, 0x41, 0xa7, 0x67, 0xf7, 0xea, 0x3f, },
      /* 8.427008e-01 */
    {0x8b, 0x08, 0x41, 0xa7, 0x67, 0xf7, 0xea, 0xbf, },
      /* -8.427008e-01 */
    {0x6d, 0x9b, 0x42, 0x50, 0xd7, 0x0d, 0x12, 0x00, },
      /* 2.510727e-308 */
    {0x6d, 0x9b, 0x42, 0x50, 0xd7, 0x0d, 0x12, 0x80, },
      /* -2.510727e-308 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x7f, },
      /* nan */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xff, },
      /* nan */
    {0xe6, 0xa2, 0xc8, 0xa6, 0x45, 0x80, 0xe9, 0x3f, },
      /* 7.969082e-01 */
    {0xc0, 0x02, 0x73, 0xd2, 0xa3, 0x2a, 0xec, 0x3f, },
      /* 8.802051e-01 */
    {0xe6, 0xa2, 0xc8, 0xa6, 0x45, 0x80, 0xe9, 0xbf, },
      /* -7.969082e-01 */
    {0xc0, 0x02, 0x73, 0xd2, 0xa3, 0x2a, 0xec, 0xbf, },
      /* -8.802051e-01 */
  };

  for (i = 0; i < 16; ++i) {
    result.v = erf( args[i * 1 + 0].v);
    if (memcmp(result.b, expected[i].b, sizeof(double)) != 0) {
      int j;
      fprintf(stderr, "\nreturned: ");
      for (j = 0; j < sizeof(double); ++j) {
        fprintf(stderr, "%02x, ", result.b[j]);
      }
      fprintf(stderr, "(%e)\n", result.v);
      fprintf(stderr, "expected: ");
      for (j = 0; j < sizeof(double); ++j) {
        fprintf(stderr, "%02x, ", expected[i].b[j]);
      }
      fprintf(stderr, "(%e)\n", expected[i].v);
      ok = 0;
    }
  }

  TEST(ok);
}
