/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/erfc.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname erfc_1
  * @testfor erfc
  */
TEST_CASE(erfc_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(double)];
    double v;
  } result, args[] = {
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, },
      /* 2.225074e-308 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, },
      /* -2.225074e-308 */
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0x7f, },
      /* 1.797693e+308 */
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0xff, },
      /* -1.797693e+308 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xff, },
      /* -inf */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x7f, },
      /* nan */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xff, },
      /* nan */
    {0xcd, 0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0xec, 0x3f, },
      /* 9.000000e-01 */
    {0x9a, 0x99, 0x99, 0x99, 0x99, 0x99, 0xf1, 0x3f, },
      /* 1.100000e+00 */
    {0xcd, 0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0xec, 0xbf, },
      /* -9.000000e-01 */
    {0x9a, 0x99, 0x99, 0x99, 0x99, 0x99, 0xf1, 0xbf, },
      /* -1.100000e+00 */
  }, expected[] = {
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f, },
      /* 1.000000e+00 */
    {0xd5, 0xdd, 0xfb, 0x62, 0x61, 0x22, 0xc4, 0x3f, },
      /* 1.572992e-01 */
    {0x45, 0x84, 0xa0, 0xd3, 0xb3, 0x7b, 0xfd, 0x3f, },
      /* 1.842701e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, },
      /* 2.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, },
      /* 2.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x7f, },
      /* nan */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xff, },
      /* nan */
    {0x69, 0x74, 0xdd, 0x64, 0xe9, 0xfe, 0xc9, 0x3f, },
      /* 2.030918e-01 */
    {0x00, 0xea, 0x67, 0x6c, 0xe1, 0xaa, 0xbe, 0x3f, },
      /* 1.197949e-01 */
    {0x73, 0x51, 0x64, 0xd3, 0x22, 0xc0, 0xfc, 0x3f, },
      /* 1.796908e+00 */
    {0x60, 0x81, 0x39, 0xe9, 0x51, 0x15, 0xfe, 0x3f, },
      /* 1.880205e+00 */
  };

  for (i = 0; i < 16; ++i) {
    result.v = erfc( args[i * 1 + 0].v);
    if (memcmp(result.b, expected[i].b, sizeof(double)) != 0) {
      int j;
      fprintf(stderr, "\nreturned: ");
      for (j = 0; j < sizeof(double); ++j) {
        fprintf(stderr, "%02x, ", result.b[j]);
      }
      fprintf(stderr, "(%e)\n", result.v);
      fprintf(stderr, "expected: ");
      for (j = 0; j < sizeof(double); ++j) {
        fprintf(stderr, "%02x, ", expected[i].b[j]);
      }
      fprintf(stderr, "(%e)\n", expected[i].v);
      ok = 0;
    }
  }

  TEST(ok);
}
