/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/frexpf.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname frexpf_1
  * @testfor frexpf
  */
TEST_CASE(frexpf_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(float)];
    float v;
  } result, args[] = {
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x80, 0x00, },
      /* 1.175494e-38 */
    {0x00, 0x00, 0x80, 0x80, },
      /* -1.175494e-38 */
    {0xff, 0xff, 0x7f, 0x7f, },
      /* 3.402823e+38 */
    {0xff, 0xff, 0x7f, 0xff, },
      /* -3.402823e+38 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0xff, },
      /* -inf */
    {0x00, 0x00, 0xe8, 0x42, },
      /* 1.160000e+02 */
    {0x00, 0x00, 0xe8, 0xc2, },
      /* -1.160000e+02 */
    {0x66, 0x66, 0x66, 0x3f, },
      /* 9.000000e-01 */
    {0xcd, 0xcc, 0x8c, 0x3f, },
      /* 1.100000e+00 */
    {0x66, 0x66, 0x66, 0xbf, },
      /* -9.000000e-01 */
    {0xcd, 0xcc, 0x8c, 0xbf, },
      /* -1.100000e+00 */
  }, expected[] = {
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x00, 0x3f, },
      /* 5.000000e-01 */
    {0x00, 0x00, 0x00, 0xbf, },
      /* -5.000000e-01 */
    {0x00, 0x00, 0x00, 0x3f, },
      /* 5.000000e-01 */
    {0x00, 0x00, 0x00, 0xbf, },
      /* -5.000000e-01 */
    {0xff, 0xff, 0x7f, 0x3f, },
      /* 9.999999e-01 */
    {0xff, 0xff, 0x7f, 0xbf, },
      /* -9.999999e-01 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0xff, },
      /* -inf */
    {0x00, 0x00, 0x68, 0x3f, },
      /* 9.062500e-01 */
    {0x00, 0x00, 0x68, 0xbf, },
      /* -9.062500e-01 */
    {0x66, 0x66, 0x66, 0x3f, },
      /* 9.000000e-01 */
    {0xcd, 0xcc, 0x0c, 0x3f, },
      /* 5.500000e-01 */
    {0x66, 0x66, 0x66, 0xbf, },
      /* -9.000000e-01 */
    {0xcd, 0xcc, 0x0c, 0xbf, },
      /* -5.500000e-01 */
  };
  int exp;
  int expected_exp[] = {
    0,
    0,
    1,
    1,
    -125,
    -125,
    128,
    128,
    0,
    0,
    7,
    7,
    0,
    1,
    0,
    1,
  };

  for (i = 0; i < 16; ++i) {
    result.v = frexpf( args[i * 1 + 0].v, &exp);
    if (memcmp(result.b, expected[i].b, sizeof(float)) != 0) {
      int j;
      fprintf(stderr, "\nreturned: ");
      for (j = 0; j < sizeof(float); ++j) {
        fprintf(stderr, "%02x, ", result.b[j]);
      }
      fprintf(stderr, "(%e)\n", result.v);
      fprintf(stderr, "expected: ");
      for (j = 0; j < sizeof(float); ++j) {
        fprintf(stderr, "%02x, ", expected[i].b[j]);
      }
      fprintf(stderr, "(%e)\n", expected[i].v);
      ok = 0;
    } else if (exp != expected_exp[i]) {
      fprintf(stderr, "\nreturned: %d\n", exp);
      fprintf(stderr, "expected: %d\n", expected_exp[i]);
      ok = 0;
    }
  }

  TEST(ok);
}
