/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/ilogbf.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname ilogbf_1
  * @testfor ilogbf
  */
TEST_CASE(ilogbf_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(float)];
    float v;
  } args[] = {
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x80, 0x00, },
      /* 1.175494e-38 */
    {0x00, 0x00, 0x80, 0x80, },
      /* -1.175494e-38 */
    {0xff, 0xff, 0x7f, 0x7f, },
      /* 3.402823e+38 */
    {0xff, 0xff, 0x7f, 0xff, },
      /* -3.402823e+38 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0xff, },
      /* -inf */
    {0x00, 0x00, 0xe8, 0x42, },
      /* 1.160000e+02 */
    {0x00, 0x00, 0xe8, 0xc2, },
      /* -1.160000e+02 */
    {0x66, 0x66, 0x66, 0x3f, },
      /* 9.000000e-01 */
    {0xcd, 0xcc, 0x8c, 0x3f, },
      /* 1.100000e+00 */
    {0x66, 0x66, 0x66, 0xbf, },
      /* -9.000000e-01 */
    {0xcd, 0xcc, 0x8c, 0xbf, },
      /* -1.100000e+00 */
  };
  int expected[] = {
    -2147483648,
    -2147483648,
    0,
    0,
    -126,
    -126,
    127,
    127,
    2147483647,
    2147483647,
    6,
    6,
    -1,
    0,
    -1,
    0,
  };
  int result;

  for (i = 0; i < 16; ++i) {
    result = ilogbf( args[i * 1 + 0].v);
    if (result != expected[i]) {
      int j;
      fprintf(stderr, "\nreturned: %d\n", result);
      fprintf(stderr, "expected: %d\n", expected[i]);
      ok = 0;
    }
  }

  TEST(ok);
}
