/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/lgammaf.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"
extern int signgam;

/**
  * @testname lgammaf_1
  * @testfor lgammaf
  */
TEST_CASE(lgammaf_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(float)];
    float v;
  } result, args[] = {
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x80, 0x00, },
      /* 1.175494e-38 */
    {0x00, 0x00, 0x80, 0x80, },
      /* -1.175494e-38 */
    {0xff, 0xff, 0x7f, 0x7f, },
      /* 3.402823e+38 */
    {0xff, 0xff, 0x7f, 0xff, },
      /* -3.402823e+38 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0xff, },
      /* -inf */
    {0x00, 0x00, 0xe8, 0x42, },
      /* 1.160000e+02 */
    {0x00, 0x00, 0xe8, 0xc2, },
      /* -1.160000e+02 */
    {0x66, 0x66, 0x66, 0x3f, },
      /* 9.000000e-01 */
    {0xcd, 0xcc, 0x8c, 0x3f, },
      /* 1.100000e+00 */
    {0x66, 0x66, 0x66, 0xbf, },
      /* -9.000000e-01 */
    {0xcd, 0xcc, 0x8c, 0xbf, },
      /* -1.100000e+00 */
  }, expected[] = {
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x50, 0xac, 0xae, 0x42, },
      /* 8.733655e+01 */
    {0x50, 0xac, 0xae, 0x42, },
      /* 8.733655e+01 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0xcb, 0xfa, 0xd8, 0x43, },
      /* 4.339593e+02 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x46, 0xf0, 0x87, 0x3d, },
      /* 6.637625e-02 */
    {0x0e, 0x47, 0x4c, 0xbd, },
      /* -4.987245e-02 */
    {0xab, 0xea, 0x16, 0x40, },
      /* 2.358073e+00 */
    {0x7f, 0x83, 0x11, 0x40, },
      /* 2.273651e+00 */
  };
  int expected_signgam[] = {
    1,
    1,
    1,
    1,
    1,
    -1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    -1,
    1,
  };

  for (i = 0; i < 16; ++i) {
    result.v = lgammaf( args[i * 1 + 0].v);
    if (memcmp(result.b, expected[i].b, sizeof(float)) != 0) {
      int j;
      fprintf(stderr, "\nreturned: ");
      for (j = 0; j < sizeof(float); ++j) {
        fprintf(stderr, "%02x, ", result.b[j]);
      }
      fprintf(stderr, "(%e)\n", result.v);
      fprintf(stderr, "expected: ");
      for (j = 0; j < sizeof(float); ++j) {
        fprintf(stderr, "%02x, ", expected[i].b[j]);
      }
      fprintf(stderr, "(%e)\n", expected[i].v);
      ok = 0;
    } else if (signgam != expected_signgam[i]) {
      fprintf(stderr, "\nreturned: %d\n", signgam);
      fprintf(stderr, "expected: %d\n", expected_signgam[i]);
    }
  }

  TEST(ok);
}
