/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/llrint.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname llrint_1
  * @testfor llrint
  */
TEST_CASE(llrint_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(double)];
    double v;
  } args[] = {
    {0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdf, 0x43, },
      /* 9.223372e+18 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xc3, },
      /* -9.223372e+18 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, },
      /* 2.225074e-308 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, },
      /* -2.225074e-308 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x3f, },
      /* 5.000000e-01 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xbf, },
      /* -5.000000e-01 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0xf9, 0x3f, },
      /* 1.570796e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0x09, 0x40, },
      /* 3.141593e+00 */
    {0xd2, 0x21, 0x33, 0x7f, 0x7c, 0xd9, 0x12, 0x40, },
      /* 4.712389e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0x19, 0x40, },
      /* 6.283185e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0xf9, 0xbf, },
      /* -1.570796e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0x09, 0xc0, },
      /* -3.141593e+00 */
    {0xd2, 0x21, 0x33, 0x7f, 0x7c, 0xd9, 0x12, 0xc0, },
      /* -4.712389e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0x19, 0xc0, },
      /* -6.283185e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0xe9, 0x3f, },
      /* 7.853982e-01 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0xe9, 0xbf, },
      /* -7.853982e-01 */
  };
  long long expected[] = {
    9223372036854773760LL,
    -9223372036854775808LL,
    0LL,
    0LL,
    1LL,
    -1LL,
    0LL,
    0LL,
    0LL,
    0LL,
    2LL,
    3LL,
    5LL,
    6LL,
    -2LL,
    -3LL,
    -5LL,
    -6LL,
    1LL,
    -1LL,
  };
  long long result;

  for (i = 0; i < 20; ++i) {
    result = llrint( args[i * 1 + 0].v);
    if (result != expected[i]) {
      fprintf(stderr, "\nreturned: %lld\n", result);
      fprintf(stderr, "expected: %lld\n", expected[i]);
      ok = 0;
    }
  }

  TEST(ok);
}
