/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/llrintf.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname llrintf_1
  * @testfor llrintf
  */
TEST_CASE(llrintf_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(float)];
    float v;
  } args[] = {
    {0xfe, 0xff, 0xff, 0x5e, },
      /* 9.223371e+18 */
    {0x00, 0x00, 0x00, 0xdf, },
      /* -9.223372e+18 */
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x80, 0x00, },
      /* 1.175494e-38 */
    {0x00, 0x00, 0x80, 0x80, },
      /* -1.175494e-38 */
    {0x00, 0x00, 0x00, 0x3f, },
      /* 5.000000e-01 */
    {0x00, 0x00, 0x00, 0xbf, },
      /* -5.000000e-01 */
    {0xdb, 0x0f, 0xc9, 0x3f, },
      /* 1.570796e+00 */
    {0xdb, 0x0f, 0x49, 0x40, },
      /* 3.141593e+00 */
    {0xe4, 0xcb, 0x96, 0x40, },
      /* 4.712389e+00 */
    {0xdb, 0x0f, 0xc9, 0x40, },
      /* 6.283185e+00 */
    {0xdb, 0x0f, 0xc9, 0xbf, },
      /* -1.570796e+00 */
    {0xdb, 0x0f, 0x49, 0xc0, },
      /* -3.141593e+00 */
    {0xe4, 0xcb, 0x96, 0xc0, },
      /* -4.712389e+00 */
    {0xdb, 0x0f, 0xc9, 0xc0, },
      /* -6.283185e+00 */
    {0xdb, 0x0f, 0x49, 0x3f, },
      /* 7.853982e-01 */
    {0xdb, 0x0f, 0x49, 0xbf, },
      /* -7.853982e-01 */
  };
  long long expected[] = {
    9223370937343148032LL,
    -9223372036854775808LL,
    0LL,
    0LL,
    1LL,
    -1LL,
    0LL,
    0LL,
    0LL,
    0LL,
    2LL,
    3LL,
    5LL,
    6LL,
    -2LL,
    -3LL,
    -5LL,
    -6LL,
    1LL,
    -1LL,
  };
  long long result;

  for (i = 0; i < 20; ++i) {
    result = llrintf( args[i * 1 + 0].v);
    if (result != expected[i]) {
      fprintf(stderr, "\nreturned: %lld\n", result);
      fprintf(stderr, "expected: %lld\n", expected[i]);
      ok = 0;
    }
  }

  TEST(ok);
}
