/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/lrint.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname lrint_1
  * @testfor lrint
  */
TEST_CASE(lrint_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(double)];
    double v;
  } args[] = {
    {0x00, 0x00, 0xe0, 0xff, 0xff, 0xff, 0xdf, 0x41, },
      /* 2.147484e+09 */
    {0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0xe0, 0xc1, },
      /* -2.147484e+09 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, },
      /* 2.225074e-308 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, },
      /* -2.225074e-308 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x3f, },
      /* 5.000000e-01 */
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xbf, },
      /* -5.000000e-01 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0xf9, 0x3f, },
      /* 1.570796e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0x09, 0x40, },
      /* 3.141593e+00 */
    {0xd2, 0x21, 0x33, 0x7f, 0x7c, 0xd9, 0x12, 0x40, },
      /* 4.712389e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0x19, 0x40, },
      /* 6.283185e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0xf9, 0xbf, },
      /* -1.570796e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0x09, 0xc0, },
      /* -3.141593e+00 */
    {0xd2, 0x21, 0x33, 0x7f, 0x7c, 0xd9, 0x12, 0xc0, },
      /* -4.712389e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0x19, 0xc0, },
      /* -6.283185e+00 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0xe9, 0x3f, },
      /* 7.853982e-01 */
    {0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0xe9, 0xbf, },
      /* -7.853982e-01 */
  };
  long expected[] = {
    -2147483648,
    -2147483648,
    0,
    0,
    1,
    -1,
    0,
    0,
    0,
    0,
    2,
    3,
    5,
    6,
    -2,
    -3,
    -5,
    -6,
    1,
    -1,
  };
  long result;

  for (i = 0; i < 20; ++i) {
    result = lrint( args[i * 1 + 0].v);
    if (result != expected[i]) {
      fprintf(stderr, "\nreturned: %ld\n", result);
      fprintf(stderr, "expected: %ld\n", expected[i]);
      ok = 0;
    }
  }

  TEST(ok);
}
