/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/lroundf.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname lroundf_1
  * @testfor lroundf
  */
TEST_CASE(lroundf_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(float)];
    float v;
  } args[] = {
    {0xfe, 0xff, 0xff, 0x4e, },
      /* 2.147483e+09 */
    {0x00, 0x00, 0x00, 0xcf, },
      /* -2.147484e+09 */
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x80, 0x00, },
      /* 1.175494e-38 */
    {0x00, 0x00, 0x80, 0x80, },
      /* -1.175494e-38 */
    {0x00, 0x00, 0x00, 0x3f, },
      /* 5.000000e-01 */
    {0x00, 0x00, 0x00, 0xbf, },
      /* -5.000000e-01 */
    {0xdb, 0x0f, 0xc9, 0x3f, },
      /* 1.570796e+00 */
    {0xdb, 0x0f, 0x49, 0x40, },
      /* 3.141593e+00 */
    {0xe4, 0xcb, 0x96, 0x40, },
      /* 4.712389e+00 */
    {0xdb, 0x0f, 0xc9, 0x40, },
      /* 6.283185e+00 */
    {0xdb, 0x0f, 0xc9, 0xbf, },
      /* -1.570796e+00 */
    {0xdb, 0x0f, 0x49, 0xc0, },
      /* -3.141593e+00 */
    {0xe4, 0xcb, 0x96, 0xc0, },
      /* -4.712389e+00 */
    {0xdb, 0x0f, 0xc9, 0xc0, },
      /* -6.283185e+00 */
    {0xdb, 0x0f, 0x49, 0x3f, },
      /* 7.853982e-01 */
    {0xdb, 0x0f, 0x49, 0xbf, },
      /* -7.853982e-01 */
  };
  long expected[] = {
    2147483392,
    -2147483648,
    0,
    0,
    1,
    -1,
    0,
    0,
    1,
    -1,
    2,
    3,
    5,
    6,
    -2,
    -3,
    -5,
    -6,
    1,
    -1,
  };
  long result;

  for (i = 0; i < 20; ++i) {
    result = lroundf( args[i * 1 + 0].v);
    if (result != expected[i]) {
      fprintf(stderr, "\nreturned: %ld\n", result);
      fprintf(stderr, "expected: %ld\n", expected[i]);
      ok = 0;
    }
  }

  TEST(ok);
}
