#!/bin/sh
if [ $# -lt 4 ]; then
  echo 'Few arguments' >&2
  echo 'usage: ./mathtestgen_mgr.sh type function numofarg <arguments> ...' >&2
  echo 'examples:'
  echo './mathtestgen_mgr.sh double sin 1 0.0 M_PI/2.0 M_PI 3\*M_PI/2.0'
  echo "./mathtestgen_mgr.sh float atan2f 2 1.0f '(float)sqrt(3.0)' 1.0f 1.0f"
  exit
fi

S_TYPE=$1
S_FUNC=$2
S_NUM_OF_ARGS=`expr $# - 3`
S_ARGS_PER_CALL=$3
S_NUM_OF_TESTS=`expr $S_NUM_OF_ARGS / $S_ARGS_PER_CALL`
S_CFILE=$2.gen.c
S_OFILE=$2.gen.o
S_TESTFILE=$2.test.c
shift
shift
shift

echo '' > $S_CFILE
echo '#include <stdio.h>' >> $S_CFILE
echo '#include <math.h>' >> $S_CFILE
echo '#include <float.h>' >> $S_CFILE
echo '#include <limits.h>' >> $S_CFILE
echo 'int main()' >> $S_CFILE
echo '{' >> $S_CFILE
echo '  int I, J;' >> $S_CFILE
echo "  int NUM_OF_ARGS = $S_NUM_OF_ARGS;" >> $S_CFILE
echo "  int ARGS_PER_CALL = $S_ARGS_PER_CALL;" >> $S_CFILE
echo "  int NUM_OF_TESTS = $S_NUM_OF_TESTS;" >> $S_CFILE
echo "  char FUNC[] = \"$S_FUNC\";" >> $S_CFILE
echo "  char TYPE[] = \"$S_TYPE\";" >> $S_CFILE
echo "  int TYPE_SIZE = sizeof($S_TYPE);" >> $S_CFILE
echo '  union {' >> $S_CFILE
echo "    $S_TYPE V;" >> $S_CFILE
echo "    unsigned char B[sizeof($S_TYPE)];" >> $S_CFILE
echo '  } ARGS[] = {' >> $S_CFILE
for arg in "$@"
do
  echo "    $arg," >> $S_CFILE
done
echo "  }, ARGS_COPY[$S_ARGS_PER_CALL], EXPECTED[$S_NUM_OF_TESTS];" >> $S_CFILE
echo '' >> $S_CFILE
echo '  for (I = 0; I < NUM_OF_TESTS; ++I) {' >> $S_CFILE
echo "    EXPECTED[I].V = $S_FUNC(" >> $S_CFILE
S_I=0
S_COMMA=,
while [ $S_I -lt $S_ARGS_PER_CALL ]
do
  if [ `expr $S_I + 1` -eq $S_ARGS_PER_CALL ]; then
    S_COMMA=''
  fi
  echo "      ARGS[I * ARGS_PER_CALL + $S_I].V$S_COMMA" >> $S_CFILE
  S_I=`expr $S_I + 1`
done
echo '    );' >> $S_CFILE
echo '  }' >> $S_CFILE
echo '' >> $S_CFILE
echo '  printf("/*\\n");' >> $S_CFILE
echo '  printf("   This file is written by Lepidum Co., Ltd.\\n");' >> $S_CFILE
echo '  printf("   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.\\n");' >> $S_CFILE
echo '  printf(" */\\n");' >> $S_CFILE
echo '  printf("\\n");' >> $S_CFILE
echo '  printf("/**\\n");' >> $S_CFILE
echo '  printf(" * @file math/%s.test.c\\n", FUNC);' >> $S_CFILE
echo '  printf(" */\\n");' >> $S_CFILE
echo '  printf("#define _ISOC99_SOURCE\\n");' >> $S_CFILE
echo '  printf("#include <math.h>\\n");' >> $S_CFILE
echo '  printf("#include <string.h>\\n");' >> $S_CFILE
echo '  printf("#include <stdio.h>\\n");' >> $S_CFILE
echo '  printf("#include <stdlib.h>\\n");' >> $S_CFILE
echo '  printf("#include <float.h>\\n");' >> $S_CFILE
echo '  printf("#include \\"common.h\\"\\n");' >> $S_CFILE
echo '  printf("\\n");' >> $S_CFILE
echo '  printf("/**\\n");' >> $S_CFILE
echo '  printf("  * @testname %s_1\\n", FUNC);' >> $S_CFILE
echo '  printf("  * @testfor %s\\n", FUNC);' >> $S_CFILE
echo '  printf("  */\\n");' >> $S_CFILE
echo '  printf("TEST_CASE(%s_1)\\n", FUNC);' >> $S_CFILE
echo '  printf("{\\n");' >> $S_CFILE
echo '  printf("  int i;\\n");' >> $S_CFILE
echo '  printf("  int ok = 1;\\n");' >> $S_CFILE
echo '  printf("  /* args and expected must be static variables for compile time */\\n");' >> $S_CFILE
echo '  printf("  static union {\\n");' >> $S_CFILE
echo '  printf("    unsigned char b[sizeof(%s)];\\n", TYPE);' >> $S_CFILE
echo '  printf("    %s v;\\n", TYPE);' >> $S_CFILE
echo '  printf("  } result, args[] = {\\n");' >> $S_CFILE
echo '  for (I = 0; I < NUM_OF_ARGS; ++I) {' >> $S_CFILE
echo '    printf("    {");' >> $S_CFILE
echo '    for (J = 0; J < TYPE_SIZE; ++J) {' >> $S_CFILE
echo '      printf("0x%02x, ", ARGS[I].B[J]);' >> $S_CFILE
echo '    }' >> $S_CFILE
echo '    printf("},\\n      /* %e */\\n", ARGS[I].V);' >> $S_CFILE
echo '  }' >> $S_CFILE
echo '  printf("  }, expected[] = {\\n");' >> $S_CFILE
echo '  for (I = 0; I < NUM_OF_TESTS; ++I) {' >> $S_CFILE
echo '    printf("    {");' >> $S_CFILE
echo '    for (J = 0; J < TYPE_SIZE; ++J) {' >> $S_CFILE
echo '      printf("0x%02x, ", EXPECTED[I].B[J]);' >> $S_CFILE
echo '    }' >> $S_CFILE
echo '    printf("},\\n");' >> $S_CFILE
echo '    printf("      /* %e */\\n", EXPECTED[I].V);' >> $S_CFILE
echo '  }' >> $S_CFILE
echo '  printf("  };\\n");' >> $S_CFILE
echo '  printf("\\n");' >> $S_CFILE
echo '  printf("  for (i = 0; i < %d; ++i) {\\n", NUM_OF_TESTS);' >> $S_CFILE
echo '  printf("    result.v = %s( ", FUNC);' >> $S_CFILE
echo '  for (I = 0; I < ARGS_PER_CALL; ++I) {' >> $S_CFILE
echo '    printf("args[i * %d + %d].v", ARGS_PER_CALL, I);' >> $S_CFILE
echo '    if (I + 1 < ARGS_PER_CALL) {' >> $S_CFILE
echo '      printf(",");' >> $S_CFILE
echo '    }' >> $S_CFILE
echo '  }' >> $S_CFILE
echo '  printf(");\\n");' >> $S_CFILE
echo '  printf("    if (memcmp(result.b, expected[i].b, sizeof(%s)) != 0) {\\n", TYPE);' >> $S_CFILE
echo '  printf("      int j;\\n");' >> $S_CFILE
echo '  printf("      fprintf(stderr, \\"\\\\nreturned: \\");\\n");' >> $S_CFILE
echo '  printf("      for (j = 0; j < sizeof(%s); ++j) {\\n", TYPE);' >> $S_CFILE
echo '  printf("        fprintf(stderr, \\"%%02x, \\", result.b[j]);\\n");' >> $S_CFILE
echo '  printf("      }\\n");' >> $S_CFILE
echo '  printf("      fprintf(stderr, \\"(%%e)\\\\n\\", result.v);\\n");' >> $S_CFILE
echo '  printf("      fprintf(stderr, \\"expected: \\");\\n");' >> $S_CFILE
echo '  printf("      for (j = 0; j < sizeof(%s); ++j) {\\n", TYPE);' >> $S_CFILE
echo '  printf("        fprintf(stderr, \\"%%02x, \\", expected[i].b[j]);\\n");' >> $S_CFILE
echo '  printf("      }\\n");' >> $S_CFILE
echo '  printf("      fprintf(stderr, \\"(%%e)\\\\n\\", expected[i].v);\\n");' >> $S_CFILE
echo '  printf("      ok = 0;\\n");' >> $S_CFILE
echo '  printf("    }\\n");' >> $S_CFILE
echo '  printf("  }\\n");' >> $S_CFILE
echo '  printf("\\n");' >> $S_CFILE
echo '  printf("  TEST(ok);\\n");' >> $S_CFILE
echo '  printf("}\\n");' >> $S_CFILE
echo '' >> $S_CFILE
echo '  return 0;' >> $S_CFILE
echo '}' >> $S_CFILE

gcc -std=c99 -O2 -D_POSIX_C_SOURCE=200112L -D_XOPEN_SOURCE=500 -lcrypt -lm -o $S_OFILE $S_CFILE
if [ $? -eq 0 ]; then
  ./$S_OFILE > $S_TESTFILE
  rm $S_OFILE
  if [ -z $NODEL ]; then
    rm $S_CFILE
  fi
else
  echo "$FUNC failed."
fi
