/*
   This file is written by Lepidum Co., Ltd.
   Copyrignt (c) 2005-2008 by Lepidum Co., Ltd.
 */

/**
 * @file math/sinhf.test.c
 */
#define _ISOC99_SOURCE
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "common.h"

/**
  * @testname sinhf_1
  * @testfor sinhf
  */
TEST_CASE(sinhf_1)
{
  int i;
  int ok = 1;
  /* args and expected must be static variables for compile time */
  static union {
    unsigned char b[sizeof(float)];
    float v;
  } result, args[] = {
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0x00, 0x00, 0x80, 0x3f, },
      /* 1.000000e+00 */
    {0x00, 0x00, 0x80, 0xbf, },
      /* -1.000000e+00 */
    {0x00, 0x00, 0x80, 0x00, },
      /* 1.175494e-38 */
    {0x00, 0x00, 0x80, 0x80, },
      /* -1.175494e-38 */
    {0xff, 0xff, 0x7f, 0x7f, },
      /* 3.402823e+38 */
    {0xff, 0xff, 0x7f, 0xff, },
      /* -3.402823e+38 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0xff, },
      /* -inf */
    {0x00, 0x00, 0xc0, 0x7f, },
      /* nan */
    {0x00, 0x00, 0xc0, 0xff, },
      /* nan */
    {0xdb, 0x0f, 0xc9, 0x3f, },
      /* 1.570796e+00 */
    {0xdb, 0x0f, 0x49, 0x40, },
      /* 3.141593e+00 */
    {0xe4, 0xcb, 0x96, 0x40, },
      /* 4.712389e+00 */
    {0xdb, 0x0f, 0xc9, 0x40, },
      /* 6.283185e+00 */
    {0xdb, 0x0f, 0xc9, 0xbf, },
      /* -1.570796e+00 */
    {0xdb, 0x0f, 0x49, 0xc0, },
      /* -3.141593e+00 */
    {0xe4, 0xcb, 0x96, 0xc0, },
      /* -4.712389e+00 */
    {0xdb, 0x0f, 0xc9, 0xc0, },
      /* -6.283185e+00 */
    {0xdb, 0x0f, 0x49, 0x3f, },
      /* 7.853982e-01 */
    {0xdb, 0x0f, 0x49, 0xbf, },
      /* -7.853982e-01 */
  }, expected[] = {
    {0x00, 0x00, 0x00, 0x00, },
      /* 0.000000e+00 */
    {0x00, 0x00, 0x00, 0x80, },
      /* -0.000000e+00 */
    {0xfe, 0x6c, 0x96, 0x3f, },
      /* 1.175201e+00 */
    {0xfe, 0x6c, 0x96, 0xbf, },
      /* -1.175201e+00 */
    {0x00, 0x00, 0x80, 0x00, },
      /* 1.175494e-38 */
    {0x00, 0x00, 0x80, 0x80, },
      /* -1.175494e-38 */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0xff, },
      /* -inf */
    {0x00, 0x00, 0x80, 0x7f, },
      /* inf */
    {0x00, 0x00, 0x80, 0xff, },
      /* -inf */
    {0x00, 0x00, 0xc0, 0x7f, },
      /* nan */
    {0x00, 0x00, 0xc0, 0xff, },
      /* nan */
    {0x7c, 0x48, 0x13, 0x40, },
      /* 2.301299e+00 */
    {0xa4, 0xc7, 0x38, 0x41, },
      /* 1.154874e+01 */
    {0x1b, 0x9e, 0x5e, 0x42, },
      /* 5.565440e+01 */
    {0x5a, 0xdf, 0x85, 0x43, },
      /* 2.677449e+02 */
    {0x7c, 0x48, 0x13, 0xc0, },
      /* -2.301299e+00 */
    {0xa4, 0xc7, 0x38, 0xc1, },
      /* -1.154874e+01 */
    {0x1b, 0x9e, 0x5e, 0xc2, },
      /* -5.565440e+01 */
    {0x5a, 0xdf, 0x85, 0xc3, },
      /* -2.677449e+02 */
    {0x39, 0x61, 0x5e, 0x3f, },
      /* 8.686710e-01 */
    {0x39, 0x61, 0x5e, 0xbf, },
      /* -8.686710e-01 */
  };

  for (i = 0; i < 22; ++i) {
    result.v = sinhf( args[i * 1 + 0].v);
    if (memcmp(result.b, expected[i].b, sizeof(float)) != 0) {
      int j;
      fprintf(stderr, "\nreturned: ");
      for (j = 0; j < sizeof(float); ++j) {
        fprintf(stderr, "%02x, ", result.b[j]);
      }
      fprintf(stderr, "(%e)\n", result.v);
      fprintf(stderr, "expected: ");
      for (j = 0; j < sizeof(float); ++j) {
        fprintf(stderr, "%02x, ", expected[i].b[j]);
      }
      fprintf(stderr, "(%e)\n", expected[i].v);
      ok = 0;
    }
  }

  TEST(ok);
}
