/*
   This file is written by Lepidum Co., Ltd.
   Copyright (c) 2005-2006 by Lepidum Co., Ltd.
 */

/**
 * @file test/net/ifaddrs.test.c
 */

#include "common.h"

#include <ifaddrs.h>
#include <string.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <net/if.h>

/**
 * @testname getifaddrs_1
 * @testfor getifaddrs
 */
TEST_CASE(getifaddrs_1)
{
  struct ifaddrs *p;
  int lo_found = 0;

  getifaddrs(&p);

  TEST_FAIL_IF(p == NULL);
  while(p){
    if(strcmp(p->ifa_name, "lo") == 0 &&
       p->ifa_addr->sa_family == AF_INET &&
       ((struct sockaddr_in*)p->ifa_addr)->sin_addr.s_addr == htonl(0x7f000001) &&
       ((struct sockaddr_in*)p->ifa_netmask)->sin_addr.s_addr == htonl(0xff000000)){
      lo_found = 1;
    }
    p = p->ifa_next;
  }

  TEST(lo_found);
}
#include <stdio.h>
/**
 * @testname getifaddrs_2
 * @testfor getifaddrs
 */
TEST_CASE(getifaddrs_2)
{
  struct ifaddrs *p;
  sa_family_t af;
  getifaddrs(&p);

  TEST_FAIL_IF(p == NULL);
  while(p){
    if (p->ifa_addr) {
      af = p->ifa_addr->sa_family;
      TEST_FAIL_IF(p->ifa_netmask && p->ifa_netmask->sa_family != af);
      /* TEST_FAIL_IF(p->ifa_broadaddr && p->ifa_broadaddr->sa_family != af); */
      TEST_FAIL_IF(p->ifa_dstaddr && p->ifa_dstaddr->sa_family != af);
    }
    p = p->ifa_next;
  }

  TEST(1);
}

/**
 * @testname if_nametoindex_1
 * @testfor if_nametoindex
 */
TEST_CASE(if_nametoindex_1)
{
  TEST_FAIL_IF(if_nametoindex("lo") == 0);
  TEST_FAIL_IF(if_nametoindex("nosuchinterface") != 0);
  TEST(1);
}
